const CHECK_MAIN_HOST_TIMEOUT = 1000 * 60 * 5;  // default 5 min
const LOAD_CANDIDATES_TIMEOUT = 60 * 1000; //  default 1 min

app.Collections.Candidates = Backbone.Collection.extend({
	model: app.Models.Candidate,
	lastSave:0,
	loadInterval: 6, //hour

	initialize: function(){
	},

	settings:{
		host: {}
	},


	setHost: function(host, reset){
		var h = host.split(':');
    	this.settings.host = {ip: h[0], port: h[1] || '443'};
    	if(this._timeoutLoad){
    		clearTimeout(this._timeoutLoad);
    		this._timeoutLoad = false;
    	}
    	this._timeoutLoad = setTimeout(jQuery.proxy(this.load, this, host, reset), LOAD_CANDIDATES_TIMEOUT); // 1 minutes
	},

	update: function(){
		if((new Date().getTime() - this.lastSave) > (this.loadInterval*60*60*1000) &&
			! this._loadXhr && ! this._timeoutLoad && this.settings.host.ip){

			let findCandidate = this.findWhere(this.settings.host);
			if (findCandidate) {
				this._timeoutLoad = setTimeout(jQuery.proxy(this.load, this, findCandidate.getHost()), LOAD_CANDIDATES_TIMEOUT); // 1 minutes
			}
		}
	},

	_timeoutLoad: false,
    _loadXhr: false,
    load: function(host, reset){
    	if(! host){
    		return false;
    	}
    	if(this._loadXhr){
    		this._loadXhr.abort();
    	}

    	if(this._timeoutLoad){
    		clearTimeout(this._timeoutLoad);
    		this._timeoutLoad = false;
    	}

    	this._loadProcess = true;

    	var models = [];
    	models.push(this.settings.host);

    	this._loadXhr = jQuery.ajax({
            url: 'http://' + host + '/api/v1/PBX/candidates' ,
            headers: {'UA': 'chromeExtension'},
            type: "GET",
            timeout: 3000,
            success: jQuery.proxy(this._loadSuccess, this, models, reset),
            error: jQuery.proxy(this._loadError, this, models, reset)
    	});
    },

    _loadSuccess: function(models, reset, response){
    	this._loadXhr = false;

    	if (response && 'result' in response && 'networkAddresses' in response.result){

    		var ips = response.result.networkAddresses;
    		var ports = response.result.ports || ['443'];

    		for(var i = 0; i < ips.length; i++){

		    	for(var j = 0; j < ports.length; j++) {
		    		if(ips[i] != this.settings.host.ip || ports[j] != this.settings.host.port){
		    			models.push({'ip': ips[i], 'port': ports[j]});
		    		}
		    	}

    		}
    		
    		if('failovers' in response.result){
    			var failovers = response.result.failovers;
    			for(var f = 0; f < failovers.length; f++){
    				
    				ips = failovers[f].networkAddresses;
    	    		ports = failovers[f].ports || ['443'];

    	    		for(var i = 0; i < ips.length; i++){

    			    	for(var j = 0; j < ports.length; j++) {
    			    		if(ips[i] != this.settings.host.ip || ports[j] != this.settings.host.port){
    			    			models.push({'ip': ips[i], 'port': ports[j], 'failover': true});
    			    		}
    			    	}

    	    		}
    			}
    		}

    		this.save(models);
    	}else if(reset || ! this.models.length){
    		this.save(models);
    	}

    	return true;
    },

    _loadError: function(models, reset, e){
    	this._loadXhr = false;

		if(e.statusText == 'abort'){
			return false;
		}

		if( ! reset && e.getResponseHeader('Server') != 'wildix-http-server'){
			this._stopSearch = false;
			this.searchOnline(function(online){
				this._loadProcess = false;
				if(online){
					this.trigger('onlineHost', online);
				}
			}.bind(this));
		}else if(reset || ! this.models.length){
			this.save(models);
		}
		return false;
	},

	ping: function() {
		if( ! this.settings.host.ip){
			return;
		}

		let hostObject = this.findWhere(this.settings.host);
		if (!hostObject)  {
			return;
		}

		// Stop load candidates due to ping
		if(this._timeoutLoad){
    		clearTimeout(this._timeoutLoad);
    		this._timeoutLoad = false;
    	}

		//Stop check main host due to ping
		if(this._timerCheckMainHost){
    		clearTimeout(this._timerCheckMainHost);
    		this._timerCheckMainHost = false;
    	}

		hostObject.ping(function(status){
    		if(status != 'pong'){
    			this._stopSearch = false
    			setTimeout(function(){
    				this.searchOnline(function(online){
    					console.log('search result: ' + online);
    					if(online){
    						this.trigger('onlineHost', online);
    					}
    				}.bind(this));
    			}.bind(this), 10000);
    		}else{
    			this.trigger('onlineHost', hostObject.getHost());
    		}
    	}.bind(this));
	},

    _numLap: 0, // number of laps - the connection attempt
    _stopSearch: false,
    searchOnline: function(responce, i, failover){
    	if(this._stopSearch){// stop search
    		this._stopSearch = false;
    		return;
    	}
    	
    	i = i || 0;
    	failover = failover === undefined ? true : failover;
    	let pbxCandidate;
    	let pbxCandidateCollection = this;
    	if(failover){
    		pbxCandidate = this.at(i);
    	}else{
			// server PBX only one
			pbxCandidate = this.findWhere({'failover': false});
    	}

    	// collection empty or no candidate
    	if( ! pbxCandidateCollection.length || pbxCandidate === undefined){
    		responce(null);
    		return;
    	}

		pbxCandidate.ping(function(status){
    		if(this._stopSearch){// stop search
        		this._stopSearch = false;
        		return;
        	}
    		if(status == 'pong'){

    			if(pbxCandidate.getHost() != this.settings.host){
    				
    				// if good ping host not equal main host, last ping main host 
    				let hostObject = this.findWhere(this.settings.host);
					if (!hostObject)  {
						return;
					}

    				var attempt = 1; // count attempt connection
    				// if no local ip - online, but domain hidden
	    			if(! pbxCandidate.isLocal() && ! pbxCandidate.isDomain() && hostObject.isDomain()){
	    				attempt = 3;
	    			}
	    			hostObject.ping(function(status){
	    				if(this._stopSearch){// stop search
	    		    		this._stopSearch = false;
	    		    		return;
	    		    	}
    		    		if(status == 'pong'){
    		    			responce(hostObject.getHost());
    		    		}else{
    		    			responce(pbxCandidate.getHost());
    		    			if(pbxCandidate.isFailover() == true){
    		    				this.timerCheckMainHost();
    		    			}
    		    		}
        		    }.bind(this), attempt); // 3 attempt
    			}else{
    				responce(pbxCandidate.getHost());
    			}
    			
    		}else{
    			i++;
    			
    			if(pbxCandidateCollection.length <= i){
    	    		if(this._numLap < 1){
    	    			i = 0;
    	    			this._numLap ++;
    	    		}else{
    	    			this._numLap = 0;
    	    			responce(null);
    	    			return;
    	    		}
    			}

    			if(i == 0){
    				setTimeout(jQuery.proxy(this.searchOnline, this, responce, i, failover), 15000);// 15 seconds
    			}else{
    				this.searchOnline(responce, i, failover);
    			}
    			
    		}
    	}.bind(this));

    },
    
    stopSearch: function(){
    	this._stopSearch = true;
    },

    _timerCheckMainHost: false,
    timerCheckMainHost: function(){
    	if(this._timerCheckMainHost){
    		clearTimeout(this._timerCheckMainHost);
    		this._timerCheckMainHost = false;
    	}
    	this._timerCheckMainHost = setTimeout(function(){
    		this.searchOnline(function(online){
				if(online){
					this.trigger('onlineHost', online);
				}else{
					this.timerCheckMainHost();
				}
			}.bind(this), 0, false);
    	}.bind(this), CHECK_MAIN_HOST_TIMEOUT) // 5 minutes
    },

    /*
     * save and restore collection from storage
     *
     * */
    uniqueArr: function(value, index, self){
		for(i=0; i<index; i++){
			if(self[i]['ip']+self[i]['port'] == value['ip']+value['port']){
				return false;
			}
		}
		return true;
	}, 
	
    save: function(models){
    	models = models.filter(this.uniqueArr);
    	this.reset(models);
		chrome.storage.sync.set({'models.candidates': models});
		this.trigger('load', this.pluck('ip'));
		this.lastSave = new Date().getTime();
    },

    fetch: function(){
		chrome.storage.sync.get('models.candidates', function(data){
			var i, arr = data['models.candidates'] || [];
			arr = arr.filter(this.uniqueArr);
			this.reset(arr);
			this.trigger('load', this.pluck('ip'));
		}.bind(this));
    }


});
