
app.Collections.Tab =  Backbone.Model.extend({
    defaults: {
        id: null,
        windowId: null,
        host: null,
        protocol: 'https:',
        path: ''
    },

    initialize: function() {

        this._setRegex();
        this._events();
    },

    _setRegex: function(candidates){

        if(candidates && candidates.length){
            this.set({regex: new RegExp('//(' + candidates.join(')|(') + ')')});
        }else{
            this.set({regex: new RegExp('/((collaboration)|(cticonnect))')});
        }
    },

    _regexInUrl: function(url){
        if(url.search(this.get('regex')) != -1){
            return true;
        }else{
            return false;
        }
    },

    loadCandidates: function(candidates){

        this._setRegex(candidates);

        if( ! this.get('id')){
            this.search();
        }
    },

    /**
     * search open tabs that match the pattern
     */
    search: function(callback){

        chrome.windows.getAll({populate:true} ,function(allWindows){

            this.reset();
            for(var i in allWindows){
                for(var j in allWindows[i].tabs){
                    if(this._regexInUrl(allWindows[i].tabs[j].url) && ( ! this.has() || ! this.isCti())){
                        this._setTab(allWindows[i].tabs[j]);
                        this.trigger('open', this.toJSON());
                    }
                }
            }
            if(typeof(callback) == 'function'){
                callback(this.get('id'));
            }

        }.bind(this));
    },

    /**
     * remember tab my host
     */
    _setTab: function(tab){

        if(tab['id'] && ( ! this.has() || ! this.isCti() || tab['id'] == this.get('id'))){

            var url = document.createElement('a');
            url.href = tab.url;

            var data = {
                    id: tab.id,
                    windowId: tab.windowId,
                    protocol: url.protocol,
                    path: url.pathname.replace(/\//g, ''),
                    host: url.host
            }

            if(( ! this.get('host') || this._regexInUrl('//' + data.host + '/' + data.path)) && (! this.has() || ( ! this.isCti() && this.isCti(data.path)) || tab['id'] == this.get('id'))){

                this.set(data);
            }
        }
    },

    /**
     * return true/false if exist tab my host
     */
    has: function(){
        return (this.get('id')) ? true : false;
    },

    /**
     * return true/false if exist tab my host and this collaboration (not wms, cdrview or..)
     */
    isCti: function(path){
        var path = path || this.get('path');
        return path == 'cticonnect' || path == 'collaboration';
    },

    /**
     * clean prop tab this class
     */
    reset: function(){
        this.set({
            id: null,
            windowId: null
        });
    },

    _events: function(){

        this.on('change:host', function(model, value, options){
            var old = this.previous('host');
            value = value.split(':')[0];
            if((value && old == null) ||
               (value != old && ! this._regexInUrl(value))
              ){
                this._setRegex([value]);
            }
        }.bind(this));

        chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab){
            if(changeInfo['status'] == 'complete'){

                if(this._regexInUrl(tab.url)){

                    this._setTab(tab);

                    if(this.get('id') == tabId){
                        if( ! this.isCti()){
                            // after load tab and tab not collaboration,
                            // return false for waiting  response
                            var j = false;
                            for(var i in this._responseWait){
                                if(this._responseWait[i]){
                                    this._responseWait[i](false);
                                    delete(this._responseWait[i]);
                                    j = true;
                                }
                            }
                            if(j === true) this.trigger('notauthorized');
                        }else{
                            // after load tab, send message in content script
                            for(var i in this._responseWait){
                                if(this._responseWait[i]){
                                    this.sendMessage({cmd: i});
                                }
                            }
                        }

                        this.trigger('open', this.toJSON());
                    }
                }else if(this.has() && tab.id == this.get('id')){
                    this.search();
                }
            }else if (changeInfo['status'] && ! this.has() && this._regexInUrl(tab.url)) {
                //
                this._setTab(tab);
            }
        }.bind(this));

        chrome.tabs.onReplaced.addListener(function(to, from){
            if((this.get('id') && from == this.get('id')) || ! this.get('id')){
                this.reset();
                chrome.tabs.get(to, function(tab){
                    if(this._regexInUrl(tab.url)){
                        this._setTab(tab);
                    }
                }.bind(this));
            }

        }.bind(this));

        chrome.tabs.onRemoved.addListener(function(tabId, removeInfo){
            if(this.has() && tabId == this.get('id')){
                // search tab, after close my tab
                this.trigger('close');
                this.search();
            }
        }.bind(this));

        chrome.tabs.onAttached.addListener(function(tabId, attachInfo){
            if(this.has() && tabId == this.get('id')){
                this.set({'windowId': attachInfo.newWindowId });
            }
        }.bind(this));

        // message from content script my tab
        chrome.runtime.onMessage.addListener(function(request, sender, callback){
            if(sender && sender['tab'] && sender.tab.id == this.get('id')){
                var cmd;
                if(typeof(request) == 'string'){
                    cmd = request;
                }else{
                    cmd = request.cmd;
                }

                if(this._responseWait[cmd]){
                    this._responseWait[cmd](request['data']);
                    delete(this._responseWait[cmd]);
                }
            }

        }.bind(this));


    },

    /**
     * focused my tab if exists
     * else create tab if exists host
     * if not exists host open window settings
     *
     * if createBackground, not focused tab
     */
    _statusCreate: false, // for fix create many tab
    open: function(createBackground){

        if(this.has()){
            chrome.windows.update(this.get('windowId'), {focused: true}, function(win){
                if(win){
                    chrome.tabs.update(this.get('id'), {selected: true}, function(tab){
                        if( ! tab){
                            this.reset();
                            this.open();
                        }
                    }.bind(this));
                }else{
                    this.reset();
                    this.open();
                }
            }.bind(this));
        }else if(this.get('host')){

            if( ! this._statusCreate){

                this._statusCreate = true;// for fix create many tab

                chrome.tabs.create({
                    url: this.get('protocol') + '//' + this.get('host') + '/collaboration/',
                    active: createBackground ? false : true
                }, function(tab){
                    this._setTab(tab);
                    this._statusCreate = false;
                }.bind(this));
            }

        }else{

            chrome.runtime.openOptionsPage();
        }
    },

    /**
     * send message for tab
     * if not my tab, create in background my tab
     *
     * message for content script
     * response
     */
    _responseWait: {},
    getAndOpen: function(message, response){

        this._responseWait[message] = response;

        if(this.has() && this.isCti()){
            this.sendMessage(message);
        }else if(this.has() && (! this.get('path') || this.get('path').slice(0,13) == 'authorization' || this.get('path').slice(0,9) == 'use_https')){
            this.trigger('notauthorized');
            response(false);
            delete(this._responseWait[message]);
        }else{
            this.open(true);
        }
    },

    /**
     * set property host
     * update regex for new host
     * search tab new host
     *
     * host
     */
    setHost: function(host, open){
        if( (! host && ! this.get('host')) ||  host != this.get('host')){

            // tab already has right host
            if(this.isOnHost(host)){
                return;
            }
            this.set({host: host});

            if(this.get('id')){
                chrome.tabs.get(this.get('id'), function(tab){
                    if(tab){
                        var url = document.createElement('a');
                        url.href = tab.url;
                        chrome.tabs.update(this.get('id'), {url: tab.url.replace(url.host, host)}, function(){
                            if(open){
                                this.open();
                            }
                        }.bind(this));
                    }else{
                        this.search(function(){
                            if(open){
                                this.open();
                            }
                        }.bind(this));
                    }
                }.bind(this));
            }else{
                this.search(function(){
                    if(open){
                        this.open();
                    }
                }.bind(this));
            }
        }
    },

    /**
     * @param {string} host Format: (ip|name):port
     * @return {boolean}
     */
    isOnHost: function (host) {
        let hostParts = host.split(':');

        return hostParts[1] === '443' && hostParts[0] === this.get('host');
    },

    /**
     * send message for my tab, if open tab
     *
     * message for content script
     * response
     */
    sendMessage: function(message, response){
        if( ! this.has() || ! this.isCti()){
            response(false);
            return false;
        }
        chrome.tabs.sendMessage(this.get('id'), message, response);
    }
});
