/**
 * main class extension
 */

var app = (function() {
	var api = {
		Models: {},
		Collections: {}, // collections or controllers

		user: null,
		host: null,
		version: 0,
		keyAppWil: 'klaamdejgopombbjfpfhlebpjlmokgna', // key wildix application
		appWil: null,
		depends: {
			call: '3.80.31100.00'
		},
		icon: {
			online: {
			    "38": chrome.extension.getURL("images/icon/48.png"),
			    "19": chrome.extension.getURL("images/icon/19.png")
			},
			offline: {
			    "38": chrome.extension.getURL("images/icon/48-offline.png"),
			    "19": chrome.extension.getURL("images/icon/19-offline.png")
			}
		},

		init: function(){
			_.each(this.Collections, function(obj, key) {
			    this[key] = new obj();
			}, this);

			// add context menu call in tabs
			chrome.contextMenus.create({
			    "title": this.t('contextMenuCall'),
			    "contexts": ['selection', 'link'],
			    "onclick" : jQuery.proxy(this._onclickContextMenu, this)
			});

			// check if install app
			chrome.management.get(this.keyAppWil, function(ext){
				if(ext && ext.enabled == true){
					// work with wildix app
					this.appWil = ext;
					this.setIcon('online');
					this.setTitle();
				}else{
					this._events();
					chrome.storage.sync.get('host', jQuery.proxy(this._start, this));
				}
				this._eventsGeneral();
			}.bind(this));
		},

		_start: function(settings, resave){
			
			if(settings['host']){

				this.Tab.setHost(settings.host, resave);
				this.Candidates.fetch();
				this.Candidates.setHost(settings.host, resave);

			}else{
				this.Tab.search();
			}

		},

		_eventsGeneral: function(){

			chrome.browserAction.onClicked.addListener(function(){
				if(this.appWil){
					chrome.management.launchApp(this.keyAppWil);
				}else{
					this.Tab.open();
				}
			}.bind(this));
			
			chrome.storage.onChanged.addListener(function(items) {
				if('host' in items){
					this._start({host: items.host.newValue}, true);
				}
			}.bind(this));

			chrome.runtime.onMessage.addListener(function(request, sender, callback){
				var cmd;
				if(typeof(request) == 'string'){
					cmd = request;
				}else{
					cmd = request.cmd;
				}
				switch(cmd){
					case 'clickToLink':
						this._onclickContextMenu({linkUrl:request.data});
						break;
					case 'existApp':
						callback(this.appWil ? true : false);
						break;
					case 'focus':
						if(request.url){
							this.focusWindowByUrl(request.url);
						}else if(!this.appWil){
							this.Tab.open();
						}
						break;
					case 'tabDisconnect':
						this._searchOnlineCandidate({target:{type:'my'}});
						break;
					case 'tabConnect':
						// tab has server host? => clear _timerOnlineCandidate / stop search candidates
						chrome.storage.sync.get('host', function (storage) {
							if (storage['host'] && storage['host'] === this.Tab.get('host')) {
								if (this._timerOnlineCandidate) {
									clearTimeout(this._timerOnlineCandidate);
									this._timerOnlineCandidate = null;
								} else {
									this.Candidates.stopSearch();
								}
							}
						}.bind(this));

						break;
						
				}
			}.bind(this));


			// events for manage app wildix
			chrome.management.onEnabled.addListener(function(info){
				if(this.keyAppWil == info.id){
					this.appWil = info;
					this.setIcon('online');
					this.setTitle();
					this._offEvents();
				}
			}.bind(this));
			chrome.management.onDisabled.addListener(function(info){
				if(this.keyAppWil == info.id){
					this.appWil = null;
					this.setIcon('offline');
					this._events();
					chrome.storage.sync.get('host', jQuery.proxy(this._start, this));
				}
			}.bind(this));

			// general search timeout on app init
			setTimeout(this._searchOnlineCandidate.bind(this,{target:{type:'my'}}), 1000);
			// search timeout on connection change
			var connection = navigator.connection || navigator.webkitConnection;
			if(connection){
				connection.addEventListener('typechange', jQuery.proxy(this._searchOnlineCandidate, this));
			}

		},
		
		_events: function(){
			
			this.Tab.on('close', jQuery.proxy(this.setIcon, this, 'offline'));
			this.Tab.on('open', jQuery.proxy(this.online, this));
			this.Tab.on('notauthorized', function(){
				this.notif(this.t('notifTitleError'), this.t('notifMesNotAuth'));
				this.Tab.open();
				return false;
			}.bind(this));

			this.Candidates.on('onlineHost', this._onlineHostHandler.bind(this));
			this.Candidates.on('load', jQuery.proxy(this.Tab.loadCandidates, this.Tab));
		},

		/**
		 * @param {string} onlineHost Format: (ip|name):port
		 * @private
		 */
		_onlineHostHandler: function (onlineHost) {
			let currentHost = this.Tab.get('host');

			// no notification & redirect if already on right host
			if (this.Tab.isOnHost(onlineHost)) {
				return;
			}

			let reloadNotification = 'reload-host';
			// clear exists reloadNotification
			chrome.notifications.getAll(allAlarms => {
				if (reloadNotification in allAlarms) {
					chrome.notifications.clear(reloadNotification);
				}
			});
			chrome.notifications.create(reloadNotification, {
				type: 'basic',
				iconUrl: chrome.runtime.getURL("images/icon/48.png"),
				title: 'Host available.',
				message: 'Reload collaboration to ' + onlineHost + '?',
				buttons: [
					{title: 'Yes, reload'}
				],
				priority: 0
			});

			chrome.notifications.onButtonClicked.addListener(notificationId => {
				if (notificationId === 'reload-host') {
					this.Tab.setHost(onlineHost);
				}
			});
		},
		
		_offEvents: function(){
			this.Tab.off('close');
			this.Tab.off('open');
			this.Tab.off('notauthorized');
			this.Candidates.off('onlineHost');
			his.Candidates.off('load');
		},

		/**
	     * update icon extension in browser
	     */
		_iconTimeout: null,
		setIcon: function(status){
			if(this.iconTimeout){
				clearTimeout(this.iconTimeout);
			}

			if( ! this.icon[status]){
				status = 'offline';
			}

			this.iconTimeout = setTimeout(function(){
				clearTimeout(this.iconTimeout);
				this.iconTimeout = null;
				chrome.browserAction.setIcon({path : this.icon[status]});
			}.bind(this), 2000);
			return true;
		},

		/**
	     * save host in storage
	     */
		setStorageHost: function(host){
			if(host && host.length){
				chrome.storage.sync.get('host', function(storage){
					if( ! storage['host']){
						chrome.storage.sync.set({host: host});
					}
				});
			}
			return true;
		},

		/**
	     * update title icon extension in browser
	     */
		setTitle: function(){
			chrome.browserAction.setTitle({
				title: this.t('extName') + (this.user ? '\n' + this.user : '') + (this.host ? '\n' + this.host : '')
			});
		},

		online: function(tab){

			this.host = tab.host;
			this.setStorageHost(tab.host);
			this.setIcon('online');
			this.setTitle();

			this.Tab.sendMessage({cmd: 'getSettings'}, function(results){
				if(results && results['user']){
					this.user = results.user;
					this.setTitle();
				}
			}.bind(this));

			this.Candidates.update();

			// start new search when new tab is opened
			this._searchOnlineCandidate({target: {type: 'my'}});
		},

		_timerOnlineCandidate: false, // timeout 5 second
		_searchOnlineCandidate: function(e){
			if(this.appWil != null){
				return;
			}
			if(this._timerOnlineCandidate){
	    		clearTimeout(this._timerOnlineCandidate);
	    		this._timerOnlineCandidate = null;
	    	}
	    	if( e.target.type != 'none'){
	    		this._timerOnlineCandidate = setTimeout(jQuery.proxy(this.Candidates.ping, this.Candidates), 5000);
	    	}

		},
		
		_formatNumberPhone: function(number){
			if(number.indexOf('(0)') > 0){// delete '(0)' if position no start string
				number = number.replace('(0)','');
			}
			number = number.replace(/[^\d_a-zA-Z\+]/g, '');
			return number;
		},

		_onclickContextMenu: function(info, tab){
			var number = '';
			if(info['linkUrl']){
				var link = info['linkUrl'].split(':');
				if( link[0] == 'tel' ||
					link[0] == 'wildix' ||
					link[0] == 'callto'){

					number = decodeURIComponent(link[1].split('?')[0]);

				}else if(info['selectionText']){
					number = this._formatNumberPhone(info.selectionText);
				}
			}else{
				number = this._formatNumberPhone(info.selectionText);
			}

			if(number.length < 2){
				this.notif(this.t('notifTitleError'), this.t('notifMesCallErrorNumber'));
				return false;
			}

			if(this.appWil){
				chrome.management.launchApp(this.keyAppWil, function(){
					setTimeout(function(){
						chrome.runtime.sendMessage(this.keyAppWil, {cmd: 'call', number: number}, function(response) {
						});
					}.bind(this), 1000); // 1 second timeout for create window in app
				}.bind(this));

			}else{
				this.actualVersion(this.depends.call, function(actualVersion){

					if( ! actualVersion){

						jQuery.ajax({
			                url: this.Tab.get('protocol') + '//' + this.host + '/api/v1/Originate/Call' ,
			                type: "POST",
			                data: {
			                	number: number
					        },
					        error: jQuery.proxy(this._errorAjaxPbx, this)
					    });
					}else{
						this.Tab.getAndOpen('getDevice', function(result){
							if(result === null){
								this.notif(this.t('notifTitleError'), this.t('notifMesCallErrorDevice'));
								this.Tab.open();
								return false;
							}else if(result){
								jQuery.ajax({
					                url: this.Tab.get('protocol') + '//' + this.host + '/api/v1/Calls/' ,
					                type: "POST",
					                data: {
					                	number: number,
					                	device: result
							        },
							        error: jQuery.proxy(this._errorAjaxPbx, this)
							    });
							}

						}.bind(this));
					}
				}.bind(this));
			}

			return true;
		},

		_errorAjaxPbx: function(e){
        	if(e && e['status'] == 401){
        		this.Tab.trigger('notauthorized');
        	}else{
        		this.notif(this.t('notifTitleError'), this.t('notifMesCallError'));
        	}
		},
		
		/*
		 * focus on window by url
		 * 
		 * */
		focusWindowByUrl: function(url){
			chrome.windows.getAll({populate:true} ,function(allWindows){
	    		for(var i in allWindows){
	    			for(var j in allWindows[i].tabs){
	    				if(allWindows[i].tabs[j].url == url){
	    					chrome.windows.update(allWindows[i].id, {focused: true}, function(win){
	    						if(win){
	    							chrome.tabs.update(allWindows[i].tabs[j].id, {selected: true}, function(tab){});
	    						}
	    					});
	    				}
	    			}
	    		}

	    	}.bind(this));
		},

		/**
		 * Compare the version with the version pbx
		 *
		 * version
		 * response(
		 * 			true - version newer
		 * 			false - version older)
		 */
		actualVersion: function(version, response){

			if( ! this.version){
				this.Tab.getAndOpen('getVersion', function(result){
					if(result){
						result = result.replace(/[^\.\d]/g,'');
						if(result){
							this.version = result;
							this.actualVersion(version, response);
						}
					}
				}.bind(this));
				return;
			}else if (this.version == version){
				response(true);
				return;
			}
			var version1 = this.version.split('.');
            var version2 = version.split('.');
            for (var i = 0; i < version1.length && i < version2.length; ++i) {
                if (i >= version1.length) {
                	response(false);
                	return;
                }
                if (i >= version2.length) {
                	response(true);
                	return;
                }
                var number1 = parseInt(version1[i], 10);
                var number2 = parseInt(version2[i], 10);
                if (number2 < number1) {
                	response(true);
                	return;
                } else if (number2 > number1) {
                	response(false);
                	return;
                }
            }
            response(false);
            return;
		},

		/**
		 * show notification html5
		 */
		notif: function(title, mess, img){
			if( ! img){
				img = chrome.extension.getURL("images/icon/48.png");
			}

			return chrome.notifications.create({
				type: 'basic',
				iconUrl: img,
				message: mess,
				title: title
			});
		},

		/**
		 * translate
		 */
		t: function(key){
			return chrome.i18n.getMessage(key);
		}
	};

	api.setIcon('offline');

	return api;
})();
